#!/bin/bash

# Script for auto updating the helium miner.

# Set default values
MINER=miner
REGION=$(cat /root/.func/miner_region)
GWPORT=1680
MINERPORT=44158
DATADIR=/root/miner_data
LOGDIR=
QUAY_URL='https://quay.io/api/v1/repository/team-helium/miner/tag/?limit=20&page=1&onlyActiveTags=true'
heltec_url='http://minerback.heltec.cn/download/patch/miner_version'

# Read switches to override any default values for non-standard configs
while getopts n:g:p:d:l:r: flag
do
   case "${flag}" in
      n) MINER=${OPTARG};;
      g) GWPORT=${OPTARG};;
      p) MINERPORT=${OPTARG};;
      d) DATADIR=${OPTARG};;
      l) LOGDIR=${OPTARG};;
      r) REGION=${OPTARG};;
      *) echo "Exiting"; exit;;
   esac
done

# Autodetect running image version and set arch
if [ "$(uname -m)" == "x86_64" ]; then
        ARCH=amd
else
        ARCH=arm
fi

running_image=$(docker container inspect -f '{{.Config.Image}}' "$MINER" | awk -F: '{print $2}')

miner_quay=$(curl -s "$QUAY_URL" --write-out '\nHTTP_Response:%{http_code}')

miner_response=$(echo "$miner_quay" | grep "HTTP_Response" | cut -d":" -f2)

if [[ $miner_response -ne 200 ]];
	then
	echo "Bad Response from Server, get lateset minerversion form heltec api"
	res=$(curl -H 'Cache-Control: no-cache'  -s --connect-timeout 10 "http://47.243.108.19/download/patch/miner_version")
	get_version_from_heltec=$(echo $res|jq .use)
	miner_latest=$(echo $res|jq .version|awk -F'"' '{print $2}')
	if [ "$miner_latest" = "" ]
	then
		echo "no version getted"
		exit
	fi
	tmp=$(echo $miner_latest|grep miner-arm64)
	if [ "$tmp" = "" ]
	then
		echo "version error"
		exit
	fi
else
miner_arm=$(echo "$miner_quay" | grep -v HTTP_Response | jq  --arg ARCH "$ARCH"   '[ .tags[] | select( .name | contains($ARCH)and contains("miner")) ]')
#echo $miner_arm | jq .
latest=$(echo "$miner_quay" | grep -v HTTP_Response | jq  --arg ARCH "$ARCH"   '[ .tags[] | select( .name | contains($ARCH)and contains("latest")) ]')

#echo $latest | jq .
latest_id=$(echo $latest | jq .[0].manifest_digest)
#echo $latest_id
index_tmp=0
while true
do
        miner_id=$(echo $miner_arm | jq .[$index_tmp].manifest_digest)
        if [ "$miner_id" = "$latest_id" ]
        then
                miner_latest=$(echo $miner_arm | jq .[$index_tmp].name | cut -d'"' -f2)
                break
        fi

        if [ "$miner_id" = "null" ]
        then
                echo  "images list end"
                exit
        fi
        	index_tmp=`expr $index_tmp + 1`
done
fi

echo $miner_latest


#echo "$0 starting with MINER=$MINER GWPORT=$GWPORT MINERPORT=$MINERPORT DATADIR=$DATADIR LOGDIR=$LOGDIR REGION=$REGION ARCH=$ARCH running_image=$running_image miner_latest=$miner_latest"

if [ "$miner_latest" = "$running_image" ];
then    echo "Miner image is already on the latest version"
        exit 0
else
         echo "find new version "$miner_latest
fi

# Pull the new miner image. Downloading it now will minimize miner downtime after stop.
echo "pull quay.io/team-helium/miner:"$miner_latest
docker pull quay.io/team-helium/miner:"$miner_latest"


res=$(docker images | grep "$miner_latest")
while [ "$res" = "" ];
do
        echo download new image failed;
        sleep 10
        docker pull quay.io/team-helium/miner:"$miner_latest"
        res=$(docker images | grep "$miner_latest")
done

if [ ! -d $DATADIR ];then mkdir $DATADIR;fi

echo "Stopping and removing old miner"
docker stop "$MINER" && docker rm "$MINER"

echo "Deleting old miner software"

for image in $(docker images | grep "miner" | awk '{print $3}'); do
	image_cleanup=$(docker images | grep "$image" | awk '{print $2}')
	#change this to $running_image if you want to keep the last 2 images
#	echo $image_cleanup $miner_latest
	if [ "$image_cleanup" = "$miner_latest" ]; then
	       continue
        else
		res=$(docker images |grep "$image" | grep "$image" -n | tail -1 | awk -F: '{print $1}')
		if [ "$res" != "1" ]
		then
			tmp1=$(docker images |grep "$image" | head -1 | awk '{print $1}')
			tmp2=$(docker images |grep "$image" | head -1 | awk '{print $2}')
			echo $tmp1:$tmp2
			docker rmi $tmp1:$tmp2
		fi

		echo "Cleaning up: $image_cleanup"
		docker image rm "$image"
        fi
done

echo "Provisioning new miner version"

if [ -n "$LOGDIR" ];
then
	LOGMOUNT="--mount type=bind,source=$LOGDIR,target=/var/data/log"
fi

docker run -d --init --cpuset-cpus="0-3" --cpus 4 --ulimit nofile=200000:200000 --env REGION_OVERRIDE="$REGION" --restart always --publish "$GWPORT":"$GWPORT"/udp --publish "$MINERPORT":"$MINERPORT"/tcp --name "$MINER" --device /dev/i2c-1 --net host --privileged -v /var/run/dbus:/var/run/dbus --mount type=bind,source="$DATADIR",target=/var/data --mount type=bind,source=/root/miner_config/docker.config,target=/config/sys.config quay.io/team-helium/miner:"$miner_latest"
echo "miner image updated to $miner_latest"
